@@ -1,4 +1,4 @@
-# $Id: balloonplot.R 1260 2008-05-05 20:27:11Z warnes $
+# $Id: balloonplot.R 1302 2008-08-14 20:06:22Z warnes $
 
 balloonplot <- function(x,...)
   UseMethod("balloonplot",x)
@@ -77,7 +77,7 @@ balloonplot.default <- function(x,y,z,
   scale.method <- match.arg(scale.method)
   scale.range  <- match.arg(scale.range)
 
-  if( any(z < 0, na.rm=TRUE ) )
+  if( scale.method=="absolute" && any(z < 0, na.rm=TRUE )  )
     warning("z value(s) below zero detected.",
             " No balloons will be displayed for these cells.")
   
